#line 1 "Specio/Declare.pm"
package Specio::Declare;

use strict;
use warnings;

use parent 'Exporter';

our $VERSION = '0.31';

use Carp qw( croak );
use Specio::Coercion;
use Specio::Constraint::Simple;
use Specio::DeclaredAt;
use Specio::Helpers qw( install_t_sub _STRINGLIKE );
use Specio::Registry qw( internal_types_for_package register );

## no critic (Modules::ProhibitAutomaticExportation)
our @EXPORT = qw(
    anon
    any_can_type
    any_does_type
    any_isa_type
    coerce
    declare
    enum
    intersection
    object_can_type
    object_does_type
    object_isa_type
    union
);
## use critic

sub import {
    my $package = shift;

    # What the heck is this monstrosity?
    #
    # Moose version 2.0901 included a first pass at support for Specio. This
    # was based on Specio c. 0.06 when Specio itself still used
    # Moose. Unfortunately, recent changes to Specio broke this support and
    # the Moose core needs updating.
    #
    # However, stable versions of Moose have since shipped with a test that
    # attempts to test itself with Specio 0.07+. This was fine until I wanted
    # to release a non-TRIAL Specio.
    #
    # Once that's out, anyone installing Specio will cause future attempts to
    # install Moose to fail until Moose includes updated Specio support!
    # Breaking Moose is not acceptable, thus this mess.
    #
    # Note that since Moose 2.1207 this test was renamed and the Specio tests
    # actually run (and pass). We still need to leave this in here for quite
    # some time. People should be able to install Specio and then install an
    # older Moose indefinitely (or at least for a year or two).
    if (   $ENV{HARNESS_ACTIVE}
        && $0 =~ m{t[\\/]type_constraints[\\/]specio\.t$} ) {

        require Test::More;
        Test::More::plan( skip_all =>
                'These tests will not pass with this version of Specio' );
        exit 0;
    }

    my $caller = caller();

    $package->export_to_level( 1, $package, @_ );

    install_t_sub(
        $caller,
        internal_types_for_package($caller)
    );

    return;
}

sub declare {
    my $name = _STRINGLIKE(shift)
        or croak 'You must provide a name for declared types';
    my %p = @_;

    my $tc = _make_tc( name => $name, %p );

    register( scalar caller(), $name, $tc, 'exportable' );

    return $tc;
}

sub anon {
    return _make_tc(@_);
}

sub enum {
    my $name;
    $name = shift if @_ % 2;
    my %p = @_;

    require Specio::Constraint::Enum;

    my $tc = _make_tc(
        ( defined $name ? ( name => $name ) : () ),
        values     => $p{values},
        type_class => 'Specio::Constraint::Enum',
    );

    register( scalar caller(), $name, $tc, 'exportable' )
        if defined $name;

    return $tc;
}

sub object_can_type {
    my $name;
    $name = shift if @_ % 2;
    my %p = @_;

    # This cannot be loaded earlier, since it loads Specio::Library::Builtins,
    # which in turn wants to load Specio::Declare (the current module).
    require Specio::Constraint::ObjectCan;

    my $tc = _make_tc(
        ( defined $name ? ( name => $name ) : () ),
        methods    => $p{methods},
        type_class => 'Specio::Constraint::ObjectCan',
    );

    register( scalar caller(), $name, $tc, 'exportable' )
        if defined $name;

    return $tc;
}

sub object_does_type {
    my $name = shift;
    my %p    = @_;

    my $caller = scalar caller();

    # If we are being called repeatedly with a single argument, then we don't
    # want to blow up because the type has already been declared. This would
    # force the user to use t() for all calls but the first, making their code
    # pointlessly more complicated.
    unless ( keys %p ) {
        if ( my $tc = internal_types_for_package($caller)->{$name} ) {
            return $tc;
        }
    }

    require Specio::Constraint::ObjectDoes;

    my $tc = _make_tc(
        name       => $name,
        role       => ( defined $p{role} ? $p{role} : $name ),
        type_class => 'Specio::Constraint::ObjectDoes',
    );

    register( scalar caller(), $name, $tc, 'exportable' );

    return $tc;
}

sub object_isa_type {
    my $name = shift;
    my %p    = @_;

    my $caller = scalar caller();
    unless ( keys %p ) {
        if ( my $tc = internal_types_for_package($caller)->{$name} ) {
            return $tc;
        }
    }

    require Specio::Constraint::ObjectIsa;

    my $tc = _make_tc(
        name       => $name,
        class      => ( defined $p{class} ? $p{class} : $name ),
        type_class => 'Specio::Constraint::ObjectIsa',
    );

    register( $caller, $name, $tc, 'exportable' );

    return $tc;
}

sub any_can_type {
    my $name;
    $name = shift if @_ % 2;
    my %p = @_;

    # This cannot be loaded earlier, since it loads Specio::Library::Builtins,
    # which in turn wants to load Specio::Declare (the current module).
    require Specio::Constraint::AnyCan;

    my $tc = _make_tc(
        ( defined $name ? ( name => $name ) : () ),
        methods    => $p{methods},
        type_class => 'Specio::Constraint::AnyCan',
    );

    register( scalar caller(), $name, $tc, 'exportable' )
        if defined $name;

    return $tc;
}

sub any_does_type {
    my $name = shift;
    my %p    = @_;

    my $caller = scalar caller();
    unless ( keys %p ) {
        if ( my $tc = internal_types_for_package($caller)->{$name} ) {
            return $tc;
        }
    }

    require Specio::Constraint::AnyDoes;

    my $tc = _make_tc(
        name       => $name,
        role       => ( defined $p{role} ? $p{role} : $name ),
        type_class => 'Specio::Constraint::AnyDoes',
    );

    register( scalar caller(), $name, $tc, 'exportable' );

    return $tc;
}

sub any_isa_type {
    my $name = shift;
    my %p    = @_;

    my $caller = scalar caller();
    unless ( keys %p ) {
        if ( my $tc = internal_types_for_package($caller)->{$name} ) {
            return $tc;
        }
    }

    require Specio::Constraint::AnyIsa;

    my $tc = _make_tc(
        name       => $name,
        class      => ( defined $p{class} ? $p{class} : $name ),
        type_class => 'Specio::Constraint::AnyIsa',
    );

    register( scalar caller(), $name, $tc, 'exportable' );

    return $tc;
}

sub intersection {
    my $name;
    $name = shift if @_ % 2;
    my %p = @_;

    require Specio::Constraint::Intersection;

    my $tc = _make_tc(
        ( defined $name ? ( name => $name ) : () ),
        %p,
        type_class => 'Specio::Constraint::Intersection',
    );

    register( scalar caller(), $name, $tc, 'exportable' )
        if defined $name;

    return $tc;
}

sub union {
    my $name;
    $name = shift if @_ % 2;
    my %p = @_;

    require Specio::Constraint::Union;

    my $tc = _make_tc(
        ( defined $name ? ( name => $name ) : () ),
        %p,
        type_class => 'Specio::Constraint::Union',
    );

    register( scalar caller(), $name, $tc, 'exportable' )
        if defined $name;

    return $tc;
}

sub _make_tc {
    my %p = @_;

    my $class = delete $p{type_class} || 'Specio::Constraint::Simple';

    $p{constraint}        = delete $p{where}   if exists $p{where};
    $p{message_generator} = delete $p{message} if exists $p{message};
    $p{inline_generator}  = delete $p{inline}  if exists $p{inline};

    return $class->new(
        %p,
        declared_at => Specio::DeclaredAt->new_from_caller(2),
    );
}

sub coerce {
    my $to = shift;
    my %p  = @_;

    $p{coercion}         = delete $p{using}  if exists $p{using};
    $p{inline_generator} = delete $p{inline} if exists $p{inline};

    return $to->add_coercion(
        Specio::Coercion->new(
            to => $to,
            %p,
            declared_at => Specio::DeclaredAt->new_from_caller(1),
        )
    );
}

1;

# ABSTRACT: Specio declaration subroutines

__END__

#line 674
